from pytholic.bible._exceptions import NotVivaCristoReiError as _nc
from pytholic.bible._books import BibleBooks as _bb


class Bible(_bb):
    """Contains the 4 gospels of the New Jerusalem Bible Edition

    These gospels are represented by the following methods: matthew(), mark(),
    luke() and john(). They receive three arguments, the first one is for the
    chapter number, the second one for the verse number and an optional third
    argument can be passed, it represents a final verse, for example,
    Matthew 1, 1-5, that "5" would be represented by the final verse, the
    third argument.

    >>> bible = Bible()
    >>> print(bible.matthew(1, 1))
    Roll of the genealogy of Jesus Christ, son of David, son of Abraham:

    >>> print(bible.mark(1, 1, 2))
    ['1. The beginning of the gospel about Jesus Christ, the Son of God. ', '2. It is written in the prophet Isaiah: Look, I am going to send my messenger in front of you to prepare your way before you. ']
    """

    def __init__(self):
        self.viva_cristo_rei = "Viva Cristo Rei"

        if self.viva_cristo_rei != "Viva Cristo Rei":
            raise _nc


if __name__ == "__main__":
    import doctest
    doctest.testmod()


import re as _re
import time as _time
import random as _random


"""
Exceptions
"""


class __UnholyWordError(Exception):
    def __str__(self):
        print("An unholy word has been used.")


class __NumberTooBigError(Exception):
    def __str__(self):
        print("I can only convert numbers as big as 9999 :(")


class __WrongAnswerError(Exception):
    def __str__(self):
        print("It's not that hard :(")


class __SelfDestructError(Exception):
    def __str__(self):
        print("BOOOOOOOOOOOOOOOOOMMMMMMMM.")


"""
Util
"""


def __int_to_roman(num):
    """Turns an int into a string containing a roman representation of that
    number.
    """
    roman_numbers = [
        1000, 900, 500, 400,
        100, 90, 50, 40,
        10, 9, 5, 4,
        1
    ]
    normal_numbers = [
        "M", "CM", "D", "CD",
        "C", "XC", "L", "XL",
        "X", "IX", "V", "IV",
        "I"
    ]
    roman_num = ''
    i = 0
    while num > 0:
        for _ in range(num // roman_numbers[i]):
            roman_num += normal_numbers[i]
            num -= roman_numbers[i]
        i += 1
    return roman_num


"""
The cool stuff :D
"""


def sanctify(text: str) -> str:
    """It turns a given string into a latin-like version.

    The letter "u" is swapped by "v", normal numbers are swapped by roman numbers, all letters are capitalized.

    :param text: The string to be formatted.
    :type text: str
    :return: The given string, but with capital letters, v instead of u and roman numbers instead of normal numbers.
    :rtype: str
    :raises TypeError: If a value other than a string is passed as parameter.
    :raises UnholyWordError: If the string contains the words: satan, demon, baphomet, lucifer, 666 or devil.
    :raises NumberTooBig: If a number is greater than 9999.

    >>> text = "Hello World!, i'm 99 years old, good luck"
    >>> text = sanctify(text)
    >>> print(text)
    HELLO WORLD!, I'M XCIX YEARS OLD, GOOD LVCK
    """
    if type(text) != str:
        raise TypeError
    sanctified_string = ""
    forbidden_words = ["satan", "demon", "baphomet", "lucifer", "666", "devil"]
    text_format = text.lower()

    # Throws an error if any of the forbidden words are used
    for word in forbidden_words:
        if word in text_format:
            raise __UnholyWordError

    # call the function to swap normal numbers into roman ones, then it swaps
    # those numbers within the string
    numbers_list = _re.findall(r'[0-9]+', text_format)
    numbers_list = sorted(numbers_list, reverse=True, key=len)
    if numbers_list is not None:
        for number in numbers_list:
            num_int = int(number)
            if num_int > 9999:
                raise __NumberTooBigError
            roman_number = __int_to_roman(num_int)
            text_format = text_format.replace(number, roman_number)

    # swaps "u" for "v" because in latin u == v
    for letter in text_format:
        if letter == "u":
            letter = "v"

        # sums up all the characters
        sanctified_string += letter

    # Capital letters because its baseder
    return sanctified_string.upper()


def trinity():
    """
    Explanation of the doctrine of the Holy Trinity through an interactive
    little game.
    """
    while True:
        question1 = input(
            "3 variables, father, son and holy_ghost, have the same value: "
            '"God", is it possible to say that they are all "God"?\n'
            "[y] Yes\n"
            "[n] No\n"
        )

        if question1.lower() == "y":
            question2 = input(
                "One value in 3 different variables, one God in 3 different "
                "Persons, simple as.\n"
                "[y] Yes\n"
            )

            if question2.lower() == "y":
                print("🇻🇦")
                break
            else:
                for countdown in range(_random.randint(3, 10), 0, -1):
                    print("Self-destruction in ", countdown)
                    _time.sleep(1)
                raise __SelfDestructError

        elif question1.lower() == "n":
            raise __WrongAnswerError
        else:
            print("Type only y or n")
            _time.sleep(1)
            continue


if __name__ == "__main__":
    import doctest
    doctest.testmod()


class NotVivaCristoReiError(Exception):
    def __str__(self):
        print("Viva Cristo Rei!")


class ChapterDoesntExistError(Exception):
    def __str__(self):
        print("Chapter doesn't exist.")


class VerseDoesntExistError(Exception):
    def __str__(self):
        print("Verse doesn't exist.")


class StartVerseGreaterThanEndOneError(Exception):
    def __str__(self):
        print("End verse must be greater than start verse.")


class ChapterAndVerseMustBeIntError(Exception):
    def __str__(self):
        print("Chapter or verse must be Int.")


class EndVerseEqualToStartOneError(Exception):
    def __str__(self):
        print("End verse must not be equal to start verse.")



import random as _random


class Prayers:
    """Contains 12 prayers

    The prayers are: Our Father, Hail Mary, Glory Be, Apostle's Creed,
    Anima Christi, Angelus, Saint Michael, conversion of sinners,
    for a happy death, come Holy Spirit, Crux Sacra (in latin),
    Hail Holy Queen. You can also return a random prayer.
    """

    def our_father(self):
        print(
            """
            Our Father,
            Who art in heaven,
            hallowed be Thy name,
            Thy kingdom come,
            Thy will be done, on earth as it is in heaven.
            Give us this day our daily bread,
            and forgive us our trespasses
            as we forgive those who trespass against us,
            and lead us not into temptation,
            but deliver us from evil.
            Amem.
            """
        )

    def hail_mary(self):
        print(
            """
            Hail Mary, full of grace, the Lord is with thee.
            Blessed art tthou amongst women,
            and blessed is the fruit of thy womb, Jesus.
            Holy Mary, Mother of God,
            pray for us sinners,
            now and at the hour of our death.
            Amem.
            """
        )

    def glory_be(self):
        print(
            """
            Glory be to the Father,
            and to the Son,
            and to the Holy Spirit,
            as it was in the beginning,
            is now, and ever shall be,
            world without end.
            Amem.
            """
        )

    def apostles_creed(self):
        print(
            """
            I believe in God,
            the Father Almighty,
            Creator of Heaven and earth;
            and in Jesus Christ, His only Son, Our Lord,
            Who was conceived by the Holy Spirit,
            born of the Virgin Mary,
            suffered under Pontius Pilate,
            was crucified, died, and was buried.
            He descended into Hell,
            on the third day He arose again from the dead.
            He ascended into Heaven,
            and is seated at the right hand of God the Father Almighty;
            from thence He shall come to judge
            the living and the dead.
            I believe in the Holy Spirit,
            the holy Catholic Church,
            the communion of saints,
            the forgiveness of sins,
            the resurrection of the body,
            and the life everlasting.
            Amem.
            """
        )

    def anima_christi(self):
        print(
            """
            Soul of Christ, sanctify me,
            Body of Christ, save me,
            Blood of Christ, inebriate me,
            Water from the side of Christ, wash me,
            Passion of Christ, strengthen me,
            O good Jesus, hear me.
            Hide me within your wounds,
            keep me close to  you,
            defend me from the evil enemy,
            call me at the hour of my death,
            and bid me to come to you,
            to praise you with your saints,
            forever and ever.
            Amem. 
            """
        )

    def angelus(self):
        print(
            """
            The Angel of the Lord declared unto Mary.
            And she conceived by the Holy Spirit.

            Hail Mary, full of grace...

            Behold the handmaid of the Lord.
            Be it done unto me according to thy word.

            Hail Mary, full of grace...

            And the Word was made Flesh.
            And dwelt among us.

            Hail Mary, full of grace...

            Pray for us, O Holy Mother of God,
            that we may be made worthy of the promises of Christ.

            Let us pray. Pour forth, we beseech thee,
            O Lord, thy grace into our hearts, that we,
            to whom the Incarnation of Christ, thy son,
            was made known by the message of an angel,
            may by his passion and cross be brought to the glory of his
            resurrection, through the same Christ our Lord.
            Amem.
            """
        )

    def saint_michael(self):
        print(
            """
            Saint Michael the Archangel,
            defend us in battle.
            Be our protection against
            the wickedness and snares of the devil.
            May God rebuke him, we humbly pray,
            and do thou, O Prince of the heavenly host,
            by the power of God,
            cast into hell Satan and all the evil spirits
            who prowl throuhghout the world
            seeking the ruin of souls.
            Amem.
            """
        )

    def conversion_sinners(self):
        print(
            """
            Lord Jesus Christ, most merciful Savior of the world,
            we humbly beseech You, by Your most Sacred Heart,
            that all the sheep who stray out of Your fold
            may in one day be converted to You,
            the Shepherd and Bishop of their souls, who lives and reigns
            with God the Father in the unity of the Holy Spirit,
            world without end.
            Amem.
            """
        )

    def happy_death(self):
        print(
            """
            O my Lord and Savior, 
            support me in my last hour 
            by the strong arms of Thy sacraments 
            and the fragrance of thy consolations. 
            Let Thy absolving words be said over me, 
            and the holy oil sign and seal me; 
            and let Thine own body be my food and Thy blood my sprinkling; 
            and let Thy Mother Mary come to me, 
            and my angel whisper peace to me, 
            and Thy glorious saints and my own dear patrons smile on me, 
            that in and through them all I may die as I desire to live, 
            in Thy Church, in Thy faith, and in Thy love. 
            Amen.
            """
        )

    def come_holy_spirit(self):
        print(
            """
            Come Holy Spirit,
            fill the hearts of your faithful
            and kindle in them the fire of your love.
            Send forth your Spirit and they shall be created.
            And You shall renew the face of the earth.
            O, God, who by the light of the Holy Spirit,
            did instruct the hearts of the faithful,
            grant that by the same Holy Spirit we may be truly wise
            and ever enjoy His consolations,
            Through Christ Our Lord,
            Amem.
            """
        )

    def crux_sacra(self):
        print(
            """
            Crux sacra sit mihi lux
            Non draco sit mihi dux
            Vade retro Sátana numquam suade mihi vana
            Sunt mala quae libas ipse venena bibas
            """
        )

    def hail_holy_queen(self):
        print(
            """
            Hail, holy Queen, Mother of mercy, hail, our life, our sweetness and our hope. 
            To thee do we cry, poor banished children of Eve: 
            to thee do we send up our sighs, mourning and weeping in this vale of tears. 
            Turn then, most gracious Advocate, thine eyes of mercy toward us, and after this our exile, 
            show unto us the blessed fruit of thy womb, Jesus, O merciful, O loving, O sweet Virgin Mary! 
            Amen.
            """
        )

    def random_prayer(self):
        prayers_list = [
            self.our_father, self.hail_mary, self.glory_be,
            self.apostles_creed, self.anima_christi, self.angelus,
            self.saint_michael, self.conversion_sinners, self.happy_death,
            self.come_holy_spirit, self.crux_sacra, self.hail_holy_queen
        ]
        prayer = _random.choice(prayers_list)

        return prayer()
